@@ -20,14 +20,14 @@ namespace xmlpp
 {
 
 DomParser::DomParser()
-: doc_(0)
+: doc_(nullptr)
 {
   //Start with an empty document:
   doc_ = new Document();
 }
 
 DomParser::DomParser(const Glib::ustring& filename, bool validate)
-: doc_(0)
+: doc_(nullptr)
 {
   set_validate(validate);
   parse_file(filename);
@@ -50,13 +50,12 @@ void DomParser::parse_file(const Glib::ustring& filename)
 
   if(!context_)
   {
-    throw internal_error("Couldn't create parsing context\n" + format_xml_error());
+    throw internal_error("Could not create parser context\n" + format_xml_error());
   }
 
-  if(context_->directory == 0)
+  if(context_->directory == nullptr)
   {
-    char* directory = xmlParserGetDirectory(filename.c_str());
-    context_->directory = (char*) xmlStrdup((xmlChar*) directory);
+    context_->directory = xmlParserGetDirectory(filename.c_str());
   }
 
   parse_context();
@@ -74,7 +73,7 @@ void DomParser::parse_memory_raw(const unsigned char* contents, size_type bytes_
 
   if(!context_)
   {
-    throw internal_error("Couldn't create parsing context\n" + format_xml_error());
+    throw internal_error("Could not create parser context\n" + format_xml_error());
   }
 
   parse_context();
@@ -96,35 +95,41 @@ void DomParser::parse_context()
 
   if(!context_)
   {
-    throw internal_error("Context not initialized\n" + format_xml_error());
+    throw internal_error("Parser context not initialized\n" + format_xml_error());
   }
 
-  xmlParseDocument(context_);
+  const int parseError = xmlParseDocument(context_);
 
-  check_for_exception();
+  try
+  {
+    check_for_exception();
+  }
+  catch (...)
+  {
+    release_underlying(); //Free doc_ and context_
+    throw; // re-throw exception
+  }
 
-  const Glib::ustring error_str = format_xml_parser_error(context_);
+  auto error_str = format_xml_parser_error(context_);
+  if (error_str.empty() && parseError == -1)
+    error_str = "xmlParseDocument() failed.";
 
   if(!error_str.empty())
   {
     release_underlying(); //Free doc_ and context_
-
     throw parse_error(error_str);
   }
 
   doc_ = new Document(context_->myDoc);
   // This is to indicate to release_underlying that we took the
   // ownership on the doc.
-  context_->myDoc = 0;
+  context_->myDoc = nullptr;
 
   //Free the parse context, but keep the document alive so people can navigate the DOM tree:
   //TODO: Why not keep the context alive too?
   Parser::release_underlying();
-
-  check_for_exception();
 }
 
-
 void DomParser::parse_stream(std::istream& in)
 {
   release_underlying(); //Free any existing document.
@@ -133,20 +138,21 @@ void DomParser::parse_stream(std::istream& in)
   xmlResetLastError();
 
   context_ = xmlCreatePushParserCtxt(
-      0, // setting thoses two parameters to 0 force the parser
+      0, // Setting those two parameters to 0 force the parser
       0, // to create a document while parsing.
-      0,
-      0,
-      ""); // here should come the filename. I don't know if it is a problem to let it empty
+      0, // chunk
+      0, // size
+      0); // no filename for fetching external entities
 
   if(!context_)
   {
-    throw internal_error("Couldn't create parsing context\n" + format_xml_error());
+    throw internal_error("Could not create parser context\n" + format_xml_error());
   }
 
   initialize_context();
 
   //TODO: Shouldn't we use a Glib::ustring here, and some alternative to std::getline()?
+  int firstParseError = XML_ERR_OK;
   std::string line;
   while(std::getline(in, line))
   {
@@ -154,33 +160,48 @@ void DomParser::parse_stream(std::istream& in)
     // about layout in certain cases.
     line += '\n';
 
-    xmlParseChunk(context_, line.c_str(), line.size() /* This is a std::string, not a ustring, so this is the number of bytes. */, 0);
+    const int parseError = xmlParseChunk(context_, line.c_str(),
+      line.size() /* This is a std::string, not a ustring, so this is the number of bytes. */, 0);
+
+    // Save the first error code if any, but read on.
+    // More errors might be reported and then thrown by check_for_exception().
+    if (parseError != XML_ERR_OK && firstParseError == XML_ERR_OK)
+      firstParseError = parseError;
   }
 
-  xmlParseChunk(context_, 0, 0, 1);
+  const int parseError = xmlParseChunk(context_, 0, 0, 1 /* last chunk */);
+  if (parseError != XML_ERR_OK && firstParseError == XML_ERR_OK)
+    firstParseError = parseError;
 
-  check_for_exception();
+  try
+  {
+    check_for_exception();
+  }
+  catch (...)
+  {
+    release_underlying(); //Free doc_ and context_
+    throw; // re-throw exception
+  }
 
-  const Glib::ustring error_str = format_xml_parser_error(context_);
+  auto error_str = format_xml_parser_error(context_);
+  if (error_str.empty() && firstParseError != XML_ERR_OK)
+    error_str = "Error code from xmlParseChunk(): " + Glib::ustring::format(firstParseError);
 
   if(!error_str.empty())
   {
     release_underlying(); //Free doc_ and context_
-
     throw parse_error(error_str);
   }
 
   doc_ = new Document(context_->myDoc);
   // This is to indicate to release_underlying that we took the
   // ownership on the doc.
-  context_->myDoc = 0;
+  context_->myDoc = nullptr;
 
 
   //Free the parse context, but keep the document alive so people can navigate the DOM tree:
   //TODO: Why not keep the context alive too?
   Parser::release_underlying();
-
-  check_for_exception();
 }
 
 void DomParser::release_underlying()
@@ -188,7 +209,7 @@ void DomParser::release_underlying()
   if(doc_)
   {
     delete doc_;
-    doc_ = 0;
+    doc_ = nullptr;
   }
 
   Parser::release_underlying();
@@ -196,7 +217,7 @@ void DomParser::release_underlying()
 
 DomParser::operator bool() const
 {
-  return doc_ != 0;
+  return doc_ != nullptr;
 }
 
 Document* DomParser::get_document()
@@ -210,5 +231,3 @@ const Document* DomParser::get_document() const
 }
 
 } // namespace xmlpp
-
-
