@@ -8,11 +8,12 @@
 // because it temporarily undefines G_DISABLE_DEPRECATED while it includes glib.h.
 #include <glibmm/threads.h> // For Glib::Threads::Mutex. Needed until the next API/ABI break.
 
+#include "libxml++/exceptions/wrapped_exception.h"
 #include "libxml++/parsers/parser.h"
 
 #include <libxml/parser.h>
 
-#include <memory> //For auto_ptr.
+#include <memory> //For unique_ptr.
 #include <map>
 
 //TODO: See several TODOs in parser.h for changes at the next API/ABI break.
@@ -350,6 +351,10 @@ void Parser::callback_error_or_warning(MsgType msg_type, void* ctx,
       {
         parser->handleException(e);
       }
+      catch(...)
+      {
+        parser->handleException(wrapped_exception(std::current_exception()));
+      }
     }
   }
 }
@@ -371,7 +376,7 @@ void Parser::check_for_exception()
   
   if(exception_)
   {
-    std::auto_ptr<exception> tmp ( exception_ );
+    std::unique_ptr<exception> tmp(exception_);
     exception_ = nullptr;
     tmp->Raise();
   }
