@@ -106,7 +106,7 @@ int TextReader::get_attribute_count() const
 Glib::ustring TextReader::get_base_uri() const
 {
   return propertyreader->String(
-      xmlTextReaderBaseUri(impl_));
+      xmlTextReaderBaseUri(impl_), true);
 }
 
 int TextReader::get_depth() const
@@ -293,7 +293,7 @@ void TextReader::set_parser_property(
 
 Node* TextReader::get_current_node()
 {
-  xmlNodePtr node = xmlTextReaderCurrentNode(impl_);
+  auto node = xmlTextReaderCurrentNode(impl_);
   if(node)
   {
     Node::create_wrapper(node);
@@ -306,22 +306,14 @@ Node* TextReader::get_current_node()
 
 const Node* TextReader::get_current_node() const
 {
-  xmlNodePtr node = xmlTextReaderCurrentNode(impl_);
-  if(node)
-  {
-    Node::create_wrapper(node);
-    return static_cast<Node*>(node->_private);
-  }
-
-  check_for_exceptions();
-  return 0;
+  return const_cast<TextReader*>(this)->get_current_node();
 }
 
 /*
 TODO: add a private constructor to Document.
 Document* TextReader::CurrentDocument()
 {
-  xmlDocPtr doc = xmlTextReaderCurrentDoc(impl_);
+  auto doc = xmlTextReaderCurrentDoc(impl_);
   if(doc)
     return new Document(doc);
 }
@@ -329,8 +321,7 @@ Document* TextReader::CurrentDocument()
 
 Node* TextReader::expand()
 {
-  xmlNodePtr node = xmlTextReaderExpand(impl_);
-  if(node)
+  auto node = xmlTextReaderExpand(impl_);
   if(node)
   {
     Node::create_wrapper(node);
@@ -355,8 +346,8 @@ bool TextReader::is_valid() const
 
 void TextReader::setup_exceptions()
 {
-  xmlTextReaderErrorFunc func = 0;
-  void* arg = 0; 
+  xmlTextReaderErrorFunc func = nullptr;
+  void* arg = nullptr; 
 
   // We respect any other error handlers already setup:
   xmlTextReaderGetErrorHandler(impl_, &func, &arg);
@@ -369,7 +360,7 @@ void TextReader::setup_exceptions()
 
 void TextReader::on_libxml_error(void* arg, const char* msg, int severity, void* /* locator */)
 {
-  TextReader* ths = (TextReader*)arg;
+  auto ths = (TextReader*)arg;
   ths->severity_ = severity;
   ths->error_ = msg ? msg : "unknown parse error";
 }
@@ -379,12 +370,11 @@ void TextReader::check_for_exceptions() const
   if( severity_ == 0 )
     return;
     
-  TextReader* ths = const_cast<TextReader*>(this);
+  auto ths = const_cast<TextReader*>(this);
 
   int severity = severity_;
   ths->severity_ = 0;
 
-  //TODO: Offer an alternative when not using exceptions?
   if( severity == XML_PARSER_SEVERITY_ERROR )
     throw parse_error(error_);
   else if( severity == XML_PARSER_SEVERITY_VALIDITY_ERROR )
@@ -404,7 +394,7 @@ bool TextReader::PropertyReader::Bool(int value)
   if(value == -1)
     owner_.check_for_exceptions();
     
-  return value;
+  return value > 0;
 }
 
 char TextReader::PropertyReader::Char(int value)
