@@ -25,7 +25,7 @@ Schema::Schema(_xmlSchema* schema)
 }
 
 Schema::Schema(Document* document, bool embed)
-: impl_(0)
+: impl_(nullptr)
 , embedded_doc_(false)
 {
   set_document(document, embed);
@@ -48,7 +48,7 @@ void Schema::set_document(Document* document, bool embed)
   }
 
   xmlResetLastError();
-  xmlSchemaParserCtxtPtr context = xmlSchemaNewDocParserCtxt( document->cobj() );
+  auto context = xmlSchemaNewDocParserCtxt( document->cobj() );
 
   if(!context)
   {
@@ -97,7 +97,7 @@ void Schema::release_underlying()
   if(impl_)
   {
     xmlSchemaFree(impl_);
-    impl_ = 0;
+    impl_ = nullptr;
   }
 }
 
