@@ -7,15 +7,13 @@
 
 #include "libxml++/validators/dtdvalidator.h"
 #include "libxml++/dtd.h"
-#include "libxml++/nodes/element.h"
-#include "libxml++/nodes/textnode.h"
-#include "libxml++/nodes/commentnode.h"
-#include "libxml++/keepblanks.h"
+#include "libxml++/nodes/node.h"
 #include "libxml++/exceptions/internal_error.h"
+#include "libxml++/exceptions/validity_error.h"
 #include "libxml++/io/istreamparserinputbuffer.h"
 #include "libxml++/document.h"
 
-#include <libxml/parserInternals.h>//For xmlCreateFileParserCtxt().
+#include <libxml/parser.h>
 
 #include <sstream>
 #include <iostream>
@@ -24,18 +22,18 @@ namespace xmlpp
 {
 
 DtdValidator::DtdValidator()
-: dtd_(0)
+: dtd_(nullptr)
 {
 }
 
 DtdValidator::DtdValidator(const Glib::ustring& file)
-: dtd_(0)
+: dtd_(nullptr)
 {
   parse_subset("",file);
 }
 
 DtdValidator::DtdValidator(const Glib::ustring& external,const Glib::ustring& system)
-: dtd_(0)
+: dtd_(nullptr)
 {
   parse_subset(external,system);
 }
@@ -54,14 +52,15 @@ void DtdValidator::parse_file(const Glib::ustring& filename)
 void DtdValidator::parse_subset(const Glib::ustring& external,const Glib::ustring& system)
 {
   release_underlying(); // Free any existing dtd.
+  xmlResetLastError();
 
-  xmlDtd* dtd = xmlParseDTD(
+  auto dtd = xmlParseDTD(
     external.empty() ? 0 : (const xmlChar *)external.c_str(),
     system.empty() ? 0 : (const xmlChar *)system.c_str());
 
   if (!dtd)
   {
-    throw parse_error("Dtd could not be parsed");
+    throw parse_error("Dtd could not be parsed.\n" + format_xml_error());
   }
 
   Node::create_wrapper(reinterpret_cast<xmlNode*>(dtd));
@@ -78,15 +77,16 @@ void DtdValidator::parse_memory(const Glib::ustring& contents)
 
 void DtdValidator::parse_stream(std::istream& in)
 {
-  release_underlying(); //Free any existing document.
+  release_underlying(); // Free any existing dtd.
+  xmlResetLastError();
 
   IStreamParserInputBuffer ibuff( in );
 
-  xmlDtd* dtd = xmlIOParseDTD( 0, ibuff.cobj(), XML_CHAR_ENCODING_UTF8 );
+  auto dtd = xmlIOParseDTD( 0, ibuff.cobj(), XML_CHAR_ENCODING_UTF8 );
 
   if (!dtd)
   {
-    throw parse_error("Dtd could not be parsed");
+    throw parse_error("Dtd could not be parsed.\n" + format_xml_error());
   }
 
   Node::create_wrapper(reinterpret_cast<xmlNode*>(dtd));
@@ -97,18 +97,18 @@ void DtdValidator::release_underlying()
 {
   if(dtd_)
   {
-    //Make a local copy as the wrapper is destroyed first
+    //Make a local pointer to the underlying xmlDtd object as the wrapper is destroyed first.
     //After free_wrappers is called dtd_ will be invalid (e.g. delete dtd_)
-    xmlDtd* dtd=dtd_->cobj();
+    auto dtd = dtd_->cobj();
     Node::free_wrappers(reinterpret_cast<xmlNode*>(dtd));
     xmlFreeDtd(dtd);
-    dtd_ = 0;
+    dtd_ = nullptr;
   }
 }
 
 DtdValidator::operator bool() const
 {
-  return dtd_ != 0;
+  return dtd_ != nullptr;
 }
 
 Dtd* DtdValidator::get_dtd()
@@ -123,33 +123,37 @@ const Dtd* DtdValidator::get_dtd() const
 
 bool DtdValidator::validate(const Document* doc)
 {
+  if (!doc)
+  {
+    throw internal_error("Document pointer cannot be 0.");
+  }
+
+  if (!dtd_)
+  {
+    throw internal_error("No DTD to use for validation.");
+  }
+
   // A context is required at this stage only
   if (!valid_)
     valid_ = xmlNewValidCtxt();
 
   if(!valid_)
   {
-    throw internal_error("Couldn't create parsing context");
-  }
-
-  if (!doc)
-  {
-    throw internal_error("Document pointer cannot be 0");
+    throw internal_error("Couldn't create validation context");
   }
 
+  xmlResetLastError();
   initialize_valid();
 
   const bool res = (bool)xmlValidateDtd( valid_, (xmlDoc*)doc->cobj(), dtd_->cobj() );
 
-  if(res == 0)
+  if (!res)
   {
     check_for_exception();
-
-    throw validity_error("Document failed Dtd validation");
+    throw validity_error("Document failed DTD validation\n" + format_xml_error());
   }
 
   return res;
 }
 
 } // namespace xmlpp
-
